/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/******************************************************************************
 * CDC RS-232 Emulation Tutorial Instructions:
 ******************************************************************************
 * Refer to Application Note AN956 for explanation of the CDC class.
 * 
 * First take a look at Exercise_Example() and study how functions are called.
 * 
 * There are five exercises, each one has a solution in the CDC\user\solutions.
 * Scroll down and look for Exercise_01,_02,_03,_04, and _05.
 * Instructions on what to do is inside each function.
 * 
 *****************************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"

#include "system\usb\usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"
#include "user\adc.h"
#include "user\string.h"
#include "user\delays.h"
#include "user\stdlib.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
byte old_sw2,old_sw3;

char input_buffer[64];
char output_buffer[64];
int result;
float tempoutput;
int foo;
float resulto;
float resultm;
float resultd;
float indoortemp;
float outdoortemp;
//int i;
//int j;
char buf[6];
char outbuf[6];
char idbuf[6];
char odbuf[6];
char idodbuf[12];
int tempstate=1;
int datatosend;
int sendflag=0;
rom char welcome[]={"PIC18F4550 Full-Speed USB - CDC RS-232 Emulation Demo\r\n\r\n"};
rom char ansi_clrscr[]={"\x1b[2J"};         // ANSI Clear Screen Command

/** P R I V A T E  P R O T O T Y P E S ***************************************/
void BlinkUSBStatus(void);
void ServiceRequests(void);
void ftoa(char *buf, double fval, int cField, int cPlaces);
void readTemp(void);
/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
   mInitAllLEDs();
	//configure ADC
	//temperature on: RB1/AN10
	TRISAbits.TRISA3=1;
	TRISAbits.TRISA2=1;
	TRISAbits.TRISA1=1;
	TRISAbits.TRISA0=1;
	//ADCON1=0b00010000;
	LATA=0xFF;
	TRISA=0x3F;
	TRISB=0b00011111;
	//using external vref+
	//using GND as vref-
	SSPCON1=0b00000000;
	//ADCON1=0b00010000;
	CVRCON=0b00000000;

			

}//end UserInit


/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
    BlinkUSBStatus();
    // User Application USB tasks


	if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
    readTemp();    
	ServiceRequests();

}//end ProcessIO


void ServiceRequests(void)
{

	if(sendflag==1){
        if(datatosend==0x00){
			if(mUSBUSARTIsTxTrfReady()){
				putsUSBUSART(idbuf);
				sendflag=0;
        	}//end if
		}
	//outdoor
		if(datatosend==0x01){
			if(mUSBUSARTIsTxTrfReady()){
				putsUSBUSART(odbuf);
				sendflag=0;
        	}//end if
		}
	//indoor, outdoor
		if(datatosend==0x02){
			if(mUSBUSARTIsTxTrfReady()){
				idodbuf[0]=idbuf[0];
				idodbuf[1]=idbuf[1];
				idodbuf[2]=idbuf[2];
				idodbuf[3]=idbuf[3];
				idodbuf[4]=idbuf[4];
				
				idodbuf[5]=0x2c;//comma code
				
				idodbuf[6]=odbuf[0];
				idodbuf[7]=odbuf[1];
				idodbuf[8]=odbuf[2];
				idodbuf[9]=odbuf[3];
				idodbuf[10]=odbuf[4];
	
				idodbuf[11]=0;//Null terminate buf
			
				putsUSBUSART(idodbuf);
				sendflag=0;
			}//end if
    	}//end if
	}//end if (sendflag=1)


	if(getsUSBUSART(input_buffer, 64)){
		foo=mCDCGetRxLength();
		//indoor
		if(input_buffer[0]==0x00){
	        datatosend=0x00;
			sendflag=1;
		}
		//outdoor
		if(input_buffer[0]==0x01){
			datatosend=0x01;
			sendflag=1;
		}
		//indoor, outdoor
		if(input_buffer[0]==0x02){
	        datatosend=0x02;
			sendflag=1;
	    }//end if
	}//if there is anything in the buffer

}//end ServiceRequests


/*****************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;



    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_1_Off();
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            mLED_1_On();
        }
        else if(usb_device_state == POWERED_STATE)
        {
           mLED_1_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_1_Off();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_On();
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

void readTemp(void){
  	//Indoor on  AN10 RB1
	//Outdoor on AN9 RB3
  if (tempstate==0){tempstate=1;}else{tempstate=0;}
  if(tempstate==0){
	OpenADC( ADC_FOSC_RC &
			ADC_RIGHT_JUST &
			ADC_20_TAD,
			ADC_CH10 &
			ADC_INT_OFF &
			ADC_VREFPLUS_EXT &
			ADC_VREFMINUS_VSS, 
			0 );
	Delay10TCYx( 5 ); // Delay for 50TCY
	ConvertADC(); // Start conversion
	while( BusyADC() ); // Wait for completion
	indoortemp = ReadADC(); // Read result
	CloseADC(); // Disable A/D converter
	indoortemp=((indoortemp/1024)*250);
	indoortemp=(indoortemp-50);
	ftoa(idbuf,indoortemp,10,3);	
  }
  if(tempstate==1){
	OpenADC( ADC_FOSC_RC &
			ADC_RIGHT_JUST &
			ADC_20_TAD,
			ADC_CH9 &
			ADC_INT_OFF &
			ADC_VREFPLUS_EXT &
			ADC_VREFMINUS_VSS, 
			0 );
	Delay10TCYx( 5 ); // Delay for 50TCY
	ConvertADC(); // Start conversion
	while( BusyADC() ); // Wait for completion
	outdoortemp = ReadADC(); // Read result
	CloseADC(); // Disable A/D converter
	outdoortemp=((outdoortemp/1024)*250);
	outdoortemp=(outdoortemp-50);
	ftoa(odbuf,outdoortemp,10,3);		
  }

}

/******************************************************************************
*	Function Name	: ftoa(fbuf,fval, cField,cPlace)
*	Input		: fval - float argument
*			; cField - total number of characters in result (including ".")
*			: cPlace - number of digit after decimal point  
*	Output		: buf - ASCII representation of the float number "fval"
*								:        
*	Return		: 
*	Description	: Convert float to ASCII
*
	//This routine correctly formats a floating point number into it's ASCII representation.
	//The routine will insert leading spaces as necessary to satisfy the requirements of the
	//specified formatting.
	//
	//eg:
	//ftoa(buf,12.34567,5,3);		//Ask for 5 digit field including '.',  and 3 digits after.
	//After the routine, buf will contain: '12.34'.
	//
	//This allows for easier integration of displays onto the VFD.
	//
*example usage: 
* char buf[];
* ftoa(buf,1.2345,10,3);
*
* buf will contain '     1.234' 
******************************************************************************/
void ftoa (char *buf, double fval, int cField, int cPlaces)
{
	long lTemp;
	long scale;
	int i;
	char dbuf[8];			// Temporary - used and overwritten in routine
	char pbuf[8];			// Temporary - used and overwritten in routine
	char fbuf[16];			// The output string is built here
														//  and copied into the *buf buffer at the end.
	char cDigits;
	//---------------------------------------------------------------------------
	size_t szTemp;
	if (cPlaces)
	{
	  cDigits = (cField - cPlaces - 1);	//FP formatting.  Includes '.' and decimal part
	}
	else
	{
	  cDigits = cField;		//Special case for integers
	}
	USBTasks();																			//EXAMPLE USED: fval=123.456789, fp(5,5) has been set
	memset (fbuf, 0, sizeof (fbuf));		//Clear the buffer
	scale = 1;													//Set default scale value
	lTemp = (long) fval;			//convert double to long	- ltemp=123
	ltoa (lTemp, dbuf);			//convert long to ascii in dbuf
	szTemp = strlen (dbuf);		//How many digits does the number use?
																			//---------------------------------------	
/*	if ((char) szTemp < cDigits)		//Do we need leading whitespaces?
	{
		for (i=0; i < (cDigits - (int) szTemp +0); i++)
		{
		 pbuf[i] = ' ';		//Insert the reqd number of spaces into the buffer 'pbuf'
		}
		if ((fval < 0) && (fval > -1) )      //Special case for -0.1 (etc) where 'digit' is = 0, but fval is -ve.
		{
		 pbuf[i-1] = '-';		//Add -ve symbol to the main buffer
		}
		pbuf[i] = 0;											                                                               //Null terminate the spaces so strcat can find it
																			//No need to add '1' - the for loop does this for us!
		strcat(fbuf, pbuf);		//Add the spaces (and sign?) into the main buffer
	}
	else
	{*/
		if ((fval < 0) && (fval > -1) )  	//Special case for -0.1 (etc) where 'digit' is = 0, but fval is -ve.
		{
			fbuf[0] = '-';	//Add -ve symbol to the main buffer
			fbuf[1] = 0;	//Null terminate
		}
	//}
	strcat(fbuf, dbuf);			//Add the numerical part to the main buffer (after '-' and spaces if reqd)
	USBTasks();
	if (cPlaces)				//Do we need the decimal part?
	{															
		strcatpgm2ram (fbuf, ".");	//add '.' to end of main buffer
		switch (cPlaces)		// Convert fractional	
		{
			case 1:	scale = 10; break;
			case 2:	scale = 100; break;
			case 3:     scale = 1000;  break;
			case 4:	scale = 10000; break;
			case 5:     scale = 100000; break;
		}
		fval = (fval - (int) fval);               //Remove integer part... fval=0.456789

		if (fval >= 0)		// Do not lose "0" in fractional part (i.e. 0.0456)
			fval += 1;
		else
			fval -= 1;
									
		fval *= scale;
		
		if (fval >= 0)										
		 fval += 0.5f;		//  For positive round number up
		else
		 fval -= 0.5f;		// For negative round number down
										
		USBTasks();
		lTemp = (long) fval;		//convert to long integer - ltemp=45678
		if (lTemp < 0) lTemp =- lTemp;	//No negative fractions!!! (prevents -1.-234 etc)
		ltoa (lTemp, dbuf);		//copy into dbuf - dbuf='45678'
		strcat(fbuf, &dbuf[1]);		//add the fractional part to the main buffer
	}				// without "1" in dbuf[0]
	//strncpy (*buf, fbuf,(size_t)cField);	//copy the final result to the output buffer
	buf[0]=fbuf[0];
	buf[1]=fbuf[1];
	buf[2]=fbuf[2];
	buf[3]=fbuf[3];
	buf[4]=fbuf[4];
	buf[5] = 0;			//Null terminate buf
									
}	


/** EOF user.c ***************************************************************/
